class MSBMouseOverlay extends MSBListButton;

#exec texture Import File=Textures\mouseOverlay.tga Name=Mouse_Icon Mips=Off Alpha=1

var vector Mouse3D;

var float TraceLength;
var bool bHitActors;
var vector hitNorm,
           traceExt,
           traceEnd;

var vector moveVector;
var vector xAxis,yAxis,zAxis;

var bool btnOne,
         btnTwo,
         btnThree,
         bShiftDown,
         bShiftRelease,
         bDeleteDown,
         bEsc,
         bTyping;

var int joyXaxis,
        joyYaxis,
        joyUAxis,
        joyVaxis;

var Actor worldMSBTargetActor,
          hudTargetActor,
          selectedActor;    //to be implemented

var int pressedKey;
var int releasedKey;

var string TextInput;

var string ButtonPrefs;              //displayed in PreferencesWindow

var Material CursorMat;

event PostBeginPlay()
{
super.PostBeginPlay();
Mouse=self;
}

function Render(Canvas C)
{
if (!bTyping)
  {
  if ((hudTargetActor==None && (btnOne || btnTwo)) || bEsc)
    deSelectAll();
  get3DActor();
  drawMouse(C);
  deleteActors(C);

  // Actor functionality and Info-Text
  if (hudTargetActor==None && worldMSBTargetActor!=None && !worldMSBTargetActor.bStatic)
    {
    DrawInfo(C);
    if (btnOne)
      LeftBtn(C);
    if (btnTwo)
      RightBtn(C);
    }
  }
else
 {
 drawCursor(C);
 }
// Reset MouseButtons after all the Buttons have checked for Clicks
btnOne=false;
btnTwo=false;
bDeleteDown=false;
bShiftRelease=false;
bEsc=false;
releasedKey=9999;
}

function LeftBtn(Canvas C)
{
ButtonList.Remove(0,ButtonList.length);
ListButton.Remove(0,ListButton.length);
fillButtonList();
super.LeftBtn(C);
assignProperties();
deselect();

// experimental - for future shortcuts and keyboard movement of actors
if (worldMSBTargetActor.IsA('MSBActor') || worldMSBTargetActor.IsA('MSBPawn'))
  selectedActor=worldMSBTargetActor;
}

function RightBtn(Canvas C)
{
}

// ****************************************************************************
// generate actor MSBPropertyButtons (stuff that appears when clicking on actor)
// ****************************************************************************
function fillButtonList()
{
local MSBActor MSA;
local MSBPawn MSP;
local MSBLight MSL;
local int i;

MSA=MSBActor(worldMSBTargetActor);
if (MSA!=None)
  {
  for (i=0;i<MSA.Buttonlist.length;i++)
    Buttonlist[i]=MSA.ButtonList[i];
  }
else
  {
  MSP=MSBPawn(worldMSBTargetActor);
  if (MSP!=None)
    {
    for (i=0;i<MSP.Buttonlist.length;i++)
      Buttonlist[i]=MSP.ButtonList[i];
    }
  else
    {
    MSL=MSBLight(worldMSBTargetActor);
    if (MSL!=None)
      {
      for (i=0;i<MSL.Buttonlist.length;i++)
        Buttonlist[i]=MSL.ButtonList[i];
      }
    }
  }
}

// ****************************************************************************
// Assign actor MSBPropertyButtons (stuff that appears when clicking on actor)
// ****************************************************************************
function assignProperties()
{
local MSBActor MSA;
local MSBPawn MSP;
local MSBLight MSL;
local int i;

MSA=MSBActor(worldMSBTargetActor);
if (MSA!=None)
  {
  for (i=0;i<MSA.Buttonlist.length;i++)
    {
    MSBPropertyButton(ListButton[i]).Property=MSA.Properties[i];
    MSBPropertyButton(ListButton[i]).Parent=worldMSBTargetActor;
    }
  }

MSP=MSBPawn(worldMSBTargetActor);
if (MSP!=None)
  {
  for (i=0;i<MSP.Buttonlist.length;i++)
    {
    MSBPropertyButton(ListButton[i]).Property=MSP.Properties[i];
    MSBPropertyButton(ListButton[i]).Parent=worldMSBTargetActor;
    }
  }
MSL=MSBLight(worldMSBTargetActor);
if (MSL!=None)
  {
  for (i=0;i<MSL.Buttonlist.length;i++)
    {
    MSBPropertyButton(ListButton[i]).Property=MSL.Properties[i];
    MSBPropertyButton(ListButton[i]).Parent=worldMSBTargetActor;
    }
  }
}

// ****************************************************************************
// draw the Mouse Icon
// ****************************************************************************
function drawMouse(Canvas C)
{
if (xpos>C.ClipX)
  xpos=C.ClipX-10;
if (xpos<0)
  xpos=0;

if (ypos>C.ClipY)
  ypos=C.ClipY-10;
if (ypos<0)
  ypos=0;

C.SetDrawColor(255,255,255,255);
C.SetPos(xpos,ypos);
C.DrawTile(Mat,16,16,0,0,32,32);
}

function DrawCursor(Canvas C)
{
if (hudTargetActor!=None)
  {
  C.SetDrawColor(255,255,255,64);
  C.SetPos(MSBGuiOverlay(hudTargetActor).xpos+MSBGuiOverlay(hudTargetActor).width,MSBGuiOverlay(hudTargetActor).ypos);
  //mouse should not move!!
  C.DrawTile(CursorMat,64,16,0,0,32,32);
  }
}

// ****************************************************************************
//3D grabbing  - note: moveVector needs dependency to screenresolution and FOV
// ****************************************************************************
function get3DActor()
{
local float sY,sZ, fX,fY;
local vector sVec;

if (M!=None)
  {
  M.GetAxes(M.Rotation,xAxis,yAxis,zAxis);

  //int variables don't divide well
  fX=M.screenX;
  fY=M.screenY;

  //  map ScreenX between -1 and 1
  sY= (xPos/fX)*2 -1;
  sZ= (yPos/fY) * fY/fX *2 - fY/fX;

  // calculate screenvector
  sVec.Y=sY;
  sVec.X=1;   //distance from eyes to plane is believed 1
  sVec.Z=-sZ; //flip MouseY
  sVec=normal(sVec) * TraceLength;

  traceEnd=sVec;
  //transform with view Coords
  traceEnd=sVec.x * xAxis + sVec.y * yAxis + svec.Z*zAxis;

  //incorporate Player Location
  traceEnd=traceEnd+M.Location;

  worldMSBTargetActor=Trace(Mouse3D,hitnorm,traceEnd,M.Location,bHitActors,traceExt);

  if (worldMSBTargetActor!=None && ( worldMSBTargetActor.IsA('LevelInfo') || worldMSBTargetActor.IsA('TerrainInfo') || worldMSBTargetActor.bStatic) )
    worldMSBTargetActor=None;
  }
}



// ****************************************************************************
// draw Actor info
// ****************************************************************************
function DrawInfo(Canvas C)
{
  C.DrawText(worldMSBTargetActor.Location.X);
  C.SetPos(xpos,ypos+10);
  C.DrawText(worldMSBTargetActor.Location.Y);
  C.SetPos(xpos,ypos+20);
  C.DrawText(worldMSBTargetActor.Location.Z);
  C.SetPos(xpos,ypos+30);
  C.DrawText(worldMSBTargetActor.Tag);
}


// ****************************************************************************
// deselect Actors and Buttons
// ****************************************************************************
function deSelectAll()
{
local int i;
//clicking in the Open deSelects ListButtons and Stuff

Log("deSelecting All");
for (i=0;i<M.myHUD.Overlays.length;i++)
  {
  MSBGuiOverlay(M.myHUD.Overlays[i]).deSelect();
  if (MSBGuiOverlay(M.myHUD.Overlays[i]).SubLevel>0)
    M.myHUD.Overlays[i--].Destroy();
  }
}

// ****************************************************************************
// Delete Actors and Buttons
// needs: special MSBPawns functionality concerning MSBRootNodes
// ****************************************************************************
function deleteActors(Canvas C)
{
local int i;

if (bDeleteDown)
  {
  if (hudTargetActor!=None && hudTargetActor.IsA('MSBNodeButton'))
    {
    if (MSBNodeButton(hudTargetActor).Order.IsA('MSBRootNode'))
      {
      worldMSBTargetActor=MSBRootNode(MSBNodeButton(hudTargetActor).Order).BasePawn;
      clearPawn();
      worldMSBTargetActor=None;
      }
    else
      {
      MSBGuiOverlay(hudTargetActor).deSelect();
      if (MSBNodeButton(hudTargetActor).NodeIn!=None)
        {
        MSBNodeButton(hudTargetActor).NodeIn.ClearLinks();
        MSBNodeButton(hudTargetActor).NodeIn.Destroy();
        }
      if (MSBNodeButton(hudTargetActor).NodeOut!=None)
        {
        MSBNodeButton(hudTargetActor).NodeOut.ClearLinks();
        MSBNodeButton(hudTargetActor).NodeOut.Destroy();
        }
      MSBNodeButton(hudTargetActor).Order.Destroy();
      for (i=0;i<M.LevelNodes.length;i++)
        if (M.LevelNodes[i]==hudTargetActor)
          M.LevelNodes.Remove(i,1);
      hudTargetActor.Destroy();
      }
    }

  if (hudTargetActor!=None && hudTargetActor.IsA('MSBScriptNodeButton'))
    {
    MSBGuiOverlay(hudTargetActor).deSelect();
    if (MSBScriptNodeButton(hudTargetActor).NodeOut!=None)
      {
      MSBScriptNodeButton(hudTargetActor).NodeOut.ClearLinks();
      MSBScriptNodeButton(hudTargetActor).NodeOut.Destroy();
      }
   for (i=0;i<M.LevelMSBScriptNodes.length;i++)
     if (M.LevelMSBScriptNodes[i]==hudTargetActor)
       M.LevelMSBScriptNodes.Remove(i,1);
   hudTargetActor.Destroy();
   }

  else if (hudTargetActor!=None && hudTargetActor.IsA('MSBGuiOverlay') && !MSBGuiOverlay(hudTargetActor).bMainMenu)
    {
    for (i=0;i<M.LevelNodes.length;i++)
      if (M.LevelNodes[i]==hudTargetActor)
        M.LevelNodes.Remove(i,1);
    hudTargetActor.Destroy();
    }
  if (worldMSBTargetActor!=None && worldMSBTargetActor.IsA('MSBActor'))
    clearActor();
  else if (worldMSBTargetActor!=None && worldMSBTargetActor.IsA('MSBPawn'))
    clearPawn();
  else if (worldMSBTargetActor!=None && worldMSBTargetActor.IsA('Actor'))
    {
    for (i=0;i<M.LevelSpecial.length;i++)
      if (M.LevelSpecial[i]==worldMSBTargetActor)
        M.LevelSpecial.Remove(i,1);
    worldMSBTargetActor.Destroy();
    }
  else                                 // don't destroy things we don't undertand
     Log("I don't know this type of thing to remove:"@hudtargetActor@worldMSBTargetActor);
//    worldMSBTargetActor.Destroy();
  }
}


//---------------------------------------------------------------------
// cleaning up Actors and Pawns after Deletion
//---------------------------------------------------------------------

function clearPawn()
{
local string ObjectName, MapName;
local int j,i;
local MSBNodeButton NB;

for (j=0;j<M.LevelNodes.length;j++)
  {
  NB=M.LevelNodes[j];
  for (i=0;i<NB.ButtonList.length;i++)
   {
    if (Caps(NB.ButtonList[i])=="MSBPROPERTYBUTTONS.MSBPROPERTYBUTTON3D")
      {
      ObjectName=NB.Order.GetPropertyText(NB.Properties[i]);
      ObjectName=GetItemName(ObjectName);
      ObjectName-="'";
      MapName=Left(string(Level), InStr(string(Level), "."));
      ObjectName=MapName$"."$ObjectName;
      if (Actor(FindObject(ObjectName,class'Actor'))==worldMSBTargetActor)
        NB.Order.SetPropertyText(NB.Properties[i],"None");
      }
    }
   if (NB.Order.IsA('MSBRootNode') && MSBRootNode(NB.Order).BasePawn==worldMSBTargetActor)
    {
    NB.deSelect();
    if (NB.NodeIn!=None)
      {
      NB.NodeIn.ClearLinks();
      NB.NodeIn.Destroy();
      }
    if (NB.NodeOut!=None)
      {
      NB.NodeOut.ClearLinks();
      NB.NodeOut.Destroy();
      }
    NB.Order.Destroy();
    for (i=0;i<M.LevelNodes.length;i++)
      if (M.LevelNodes[i]==hudTargetActor)
        M.LevelNodes.Remove(i,1);
    NB.Destroy();
    }
  }
for (i=0;i<M.LevelPawns.length;i++)
  if (M.LevelPawns[i]==worldMSBTargetActor)
    {
    M.LevelPawns[i].clearPawn();
    M.LevelPawns.Remove(i,1);
    }
worldMSBTargetActor.Destroy();
}

function clearActor()
{
local string ObjectName, MapName;
local int j,i;
local MSBNodeButton NB;

for (j=0;j<M.LevelNodes.length;j++)
  {
  NB=M.LevelNodes[j];
  for (i=0;i<NB.ButtonList.length;i++)
   {
    if (Caps(NB.ButtonList[i])=="MSBPROPERTYBUTTONS.MSBPROPERTYBUTTON3D")
      {
      ObjectName=NB.Order.GetPropertyText(NB.Properties[i]);
      ObjectName=GetItemName(ObjectName);
      ObjectName-="'";
      MapName=Left(string(Level), InStr(string(Level), "."));
      ObjectName=MapName$"."$ObjectName;
      if (Actor(FindObject(ObjectName,class'Actor'))==worldMSBTargetActor)
        NB.Order.SetPropertyText(NB.Properties[i],"None");
      }
    }
  }
for (i=0;i<M.LevelActors.length;i++)
  if (M.LevelActors[i]==worldMSBTargetActor)
    {
    M.LevelActors[i].clearActor();
    M.LevelActors.Remove(i,1);
    }
worldMSBTargetActor.Destroy();
}

defaultproperties
{
Mat=Material'Mouse_Icon'
CursorMat=Material'Node_Icon'

TraceLength=5000
traceExt=(X=1,Y=1,Z=1)
bHitActors=true
width=32
height=32
bCircle=true
}
