class MSBInterpolationHelper extends Actor;

var MSBActor Parent;
var Actor Target;
var int keyNum;

//Posing and Rotation Vars
var MSBAniKey BaseKey;
var MSBAniKey TargetKey;
var MSBPawn MSP;

var float blendTime;
var float currentTime;

var rotator BaseRotation;
var rotator TargetRotation;

var rotator RotSpeed;
var float accelerationModRotate;
var float dampingModRotate;

//Location Vars
var vector BaseLocation;
var vector TargetLocation;

var vector MovSpeed;
var float accelerationModMov;
var float dampingModMov;

//Value Vars
var float BaseValue;
var float TargetValue;

enum EInterpolationType
{
	IT_Linear,
	IT_Spring
};

var EInterpolationType InterpolationType;

//internal
var float Value;
var float changeAmount;
var float Alpha;

var bool bLocationTween, bRotationTween, bPoseTween, bJustSpawned;
//******************************************************************************
// initializing
//******************************************************************************
function PostBeginPlay()
{
super.PostBeginPlay();
if (Owner!=None)
  Parent=MSBActor(Owner);
bJustSpawned=true;
}

function initializePoseTween()
{
bPoseTween=true;
MSP=MSBPawn(TargetKey.KeyActor);
if (MSP.PoseKey[TargetKey.BoneNumber]==None)
  MSP.iniPosing();
BaseKey=MSP.PoseKey[TargetKey.BoneNumber];
BaseRotation=MSP.PoseKey[TargetKey.BoneNumber].BoneRotation;
TargetRotation=TargetKey.BoneRotation;
bJustSpawned=false;
}

function initializeLocationTween()
{
bLocationTween=true;
if (Parent!=None)
  SetLocation(Parent.Location);
bJustSpawned=false;
}

function initializeRotationTween()
{
bRotationTween=true;
bJustSpawned=false;
}

//******************************************************************************
// Tick
//******************************************************************************
function Tick(float DeltaTime)
{
  if (!bJustSpawned && !bPoseTween && !bLocationTween && !bRotationTween)
    Destroy();
  if (bPoseTween)
    TweenPoseKey(DeltaTime);
  if (bLocationTween)
    TweenLocation(DeltaTime);
  if (bRotationTween)
    TweenRotationSpring(DeltaTime);
}


//******************************************************************************
// Rotation Tweening
//******************************************************************************
function TweenRotationSpring(float DeltaTime)
{
  local rotator accelRot;

  //if reached rotation, stop rotation tween
  if (VSize(vector(Parent.Rotation-Target.Rotation))<0.4)
    {
     Parent.SetRotation(Target.Rotation);
     bRotationTween=false;
    }
  else
    {
    accelRot = ((Target.Rotation)-Parent.Rotation) * accelerationModRotate;
    accelRot = accelRot - RotSpeed*dampingModRotate; //damping
    RotSpeed=RotSpeed+accelRot*DeltaTime;
    }
  Parent.SetRotation(Parent.Rotation+RotSpeed);
}

//******************************************************************************
// Location Tweening
//******************************************************************************
function TweenLocation(float DeltaTime)
{
  local vector accelMov;

  // We're very close to the target and might stop moving...
  if (VSize(Target.Location-Location)<10)
    {
    //check if Target is a KeyFrame and assign next KeyFrame
    if (Target==Parent.Keys[KeyNum] && KeyNum<Parent.Keys.length-1)
        Target=Parent.Keys[++KeyNum];
    //stop moving and set location of Target
    else
      {
      Parent.SetLocation(Target.Location);
      bLocationTween=false;
      }
    }
  //Move on
  else
    {
    accelMov = (Target.Location-Location) * accelerationModMov;
    accelMov = accelMov - MovSpeed*dampingModMov; //damping
    MovSpeed=MovSpeed+accelMov*DeltaTime;
    SetLocation(Location+MovSpeed);
    Parent.SetLocation(Location);
    }

}

//******************************************************************************
// PoseKey Tweening
//******************************************************************************
function TweenPoseKey(float DeltaTime)
{
local rotator RotStep;
local rotator delta;
if (currentTime>BlendTime)
  {
  bPoseTween=false;
  BaseKey.BoneRotation=TargetRotation;
  BaseKey.changeBoneRotation();
  }
else
  {
  currentTime+=DeltaTime;
  RotStep=rot(10,10,10);

  //delta=(currentTime/BlendTime)*(currentKey.BoneRotation-BaseRotation);
  delta=(currentTime/BlendTime)*(TargetRotation-BaseRotation);
  RotStep=BaseRotation+delta;
  BaseKey.BoneRotation=RotStep;
  BaseKey.changeBoneRotation();
  }
}

function TweenPoseKeySpring(float DeltaTime)
{
  local rotator rotDir;

  currentTime+=DeltaTime;
  if (currentTime>BlendTime)
    bPoseTween=false;

  rotDir=(TargetRotation-BaseKey.BoneRotation)*accelerationModRotate;
  rotDir=rotDir-dampingModRotate*rotSpeed;
  rotSpeed=rotSpeed+rotDir*DeltaTime;

  BaseKey.BoneRotation+=rotSpeed;
  BaseKey.changeBoneRotation();
}

//******************************************************************************
defaultproperties
{
     accelerationModMov=0.1;
     dampingModMov=3;

     accelerationModRotate=0.1;
     dampingModRotate=4;

     bStatic=False
     bHidden=True
     bNodelete=False
     RemoteRole=ROLE_SimulatedProxy
     blendTime=1
}
